@@ -2,22 +2,22 @@
 #include "yacasprivate.h"
 #include "genericstructs.h"
 
-GenericStruct::GenericStruct(LispCharPtr aTypeName, void* aData, void (*aDestructor)(void*))
+GenericStruct::GenericStruct(LispChar * aTypeName, void* aData, void (*aDestructor)(void*))
 :iData(aData),iTypeName(aTypeName),iDestructor(aDestructor)
 {
 }
 GenericStruct::~GenericStruct()
 {
-    if (iDestructor != NULL && iData != NULL)
+    if (iDestructor && iData)
     iDestructor(iData);
 }
 
-LispCharPtr GenericStruct::Send(LispArgList& aArgList)
+LispChar * GenericStruct::Send(LispArgList& aArgList)
 {
     return "";
 }
 
-LispCharPtr GenericStruct::TypeName()
+LispChar * GenericStruct::TypeName()
 {
     return iTypeName;
 }
