@@ -2,7 +2,6 @@
 #include "yacasbase.h"
 #include "choices.h"
 #include "lispenvironment.h"
-#include "lispplugin.h"
 #include "lispeval.h"
 #include "lispatom.h"
 #include "standard.h"
@@ -14,6 +13,10 @@
 // we need this only for digits_to_bits
 #include "numbers.h"
 
+#ifdef YACAS_DEBUG
+#include <stdio.h> // Safe, only included if YACAS_DEBUG is defined
+#endif
+
 #define InternalEval iEvaluator->Eval
 
 
@@ -30,15 +33,33 @@ LispEnvironment::LispEnvironment(
                     LispOperators &aBodiedOperators,
                     LispInput*    aCurrentInput,
                     LispInt aStackSize)
-    : 
-    iPrecision(10),	// default user precision of 10 decimal digits
-	iBinaryPrecision(34),	// same as 34 bits
+    :
+    iPrecision(10),  // default user precision of 10 decimal digits
+    iBinaryPrecision(34),  // same as 34 bits
+    iInputDirectories(),
+    iCleanup(),
     iEvalDepth(0),
     iMaxEvalDepth(1000),
     iArchive(NULL),
     iEvaluator(NEW BasicEvaluator),
+    iInputStatus(),
     iSecure(0),
+    iTrue(),
+    iFalse(),
+    iEndOfFile(),
+    iEndStatement(),
+    iProgOpen(),
+    iProgClose(),
+    iNth(),
+    iBracketOpen(),
+    iBracketClose(),
+    iListOpen(),
+    iListClose(),
+    iComma(),
+    iList(),
+    iProg(),
     iLastUniqueId(1),
+    iError(),
     iErrorOutput(iError),
     iDebugger(NULL),
     iLocalsList(NULL),
@@ -46,6 +67,7 @@ LispEnvironment::LispEnvironment(
     iCoreCommands(aCoreCommands),
     iUserFunctions(aUserFunctions),
     iHashTable(aHashTable),
+    iDefFiles(),
     iPrinter(aPrinter),
     iCurrentOutput(aOutput),
     iGlobals(aGlobals),
@@ -57,6 +79,9 @@ LispEnvironment::LispEnvironment(
     theUserError(NULL),
     iPrettyReader(NULL),
     iPrettyPrinter(NULL),
+    iDefaultTokenizer(),
+    iCommonLispTokenizer(),
+    iXmlTokenizer(),
     iCurrentTokenizer(&iDefaultTokenizer),
     iStack(aStackSize)
 {
@@ -76,41 +101,23 @@ LispEnvironment::LispEnvironment(
     iList         = LispAtom::New(*this,"List");
     iProg         = LispAtom::New(*this,"Prog");
     PushLocalFrame(LispTrue);
-    iCTokenizer.SetRemarkReceiver(*this);
 }
 
 
 LispEnvironment::~LispEnvironment()
 {
-    PopLocalFrame();
+  PopLocalFrame();
 
-    {
-
-        LispInt i,nr=iDlls.NrItems();
-#ifdef YACAS_DEBUG
-      printf("Entered environment destructor, %d DLLs loaded\n",nr);
-#endif// YACAS_DEBUG
-        for (i=0;i<nr;i++)
-        {
-#ifdef YACAS_DEBUG
-          printf("Closing DLL \"%s\"\n",iDlls[i]->DllFileName());
-#endif // YACAS_DEBUG
-            iDlls[i]->Close(*this);
-            delete iDlls[i];
-            iDlls[i] = NULL;
-        }
-    }
-    
-    LISPASSERT(iLocalsList == NULL);
-    delete iEvaluator;
-    if (iDebugger) delete iDebugger;
-    delete iArchive;
+  LISPASSERT(!iLocalsList);
+  delete iEvaluator;
+  if (iDebugger) delete iDebugger;
+  delete iArchive;
 }
 
 void LispEnvironment::SetPrecision(LispInt aPrecision)
 {
-    iPrecision = aPrecision;	// precision in decimal digits
-	iBinaryPrecision = digits_to_bits(aPrecision, BASE10);	// in bits
+    iPrecision = aPrecision;  // precision in decimal digits
+  iBinaryPrecision = digits_to_bits(aPrecision, BASE10);  // in bits
 }
 
 LispInt LispEnvironment::GetUniqueId()
@@ -119,13 +126,13 @@ LispInt LispEnvironment::GetUniqueId()
 }
 
 
-LispPtr *LispEnvironment::FindLocal(LispStringPtr aVariable)
+LispPtr *LispEnvironment::FindLocal(LispString * aVariable)
 {
-    Check(iLocalsList != NULL,KLispErrInvalidStack);
-//    Check(iLocalsList->iFirst != NULL,KLispErrInvalidStack);
+    Check(iLocalsList,KLispErrInvalidStack);
+//    Check(iLocalsList->iFirst,KLispErrInvalidStack);
     LispLocalVariable *t = iLocalsList->iFirst;
 
-    while (t != NULL)
+    while (t)
     {
         if (t->iVariable == aVariable)
         {
@@ -136,58 +143,93 @@ LispPtr *LispEnvironment::FindLocal(LispStringPtr aVariable)
     return NULL;
 }
 
-void LispEnvironment::SetVariable(LispStringPtr aVariable, LispPtr& aValue)
-{
-    LispPtr *local = FindLocal(aVariable);
-    if (local != NULL)
-    {
-        local->Set(aValue.Get());
-        return;
-    }
 
-    iGlobals.SetAssociation(LispGlobalVariable(aValue), aVariable);
+#ifdef YACAS_DEBUG
+void LispEnvironment::DebugModeVerifySettingGlobalVariables(LispPtr & aVariable, LispBoolean aGlobalLazyVariable)
+{
+  LispString *varString = aVariable->String();
+  LispPtr *local = FindLocal(varString);
+  if (local)
+  {
+    if (aGlobalLazyVariable)
+    {
+      printf("WARNING: setting local variable \"%s\" (file %s, line %d), but doing it through a method that is trying to set a global lazy variable. This is probably unintended.\n",
+        varString->c_str(),
+        aVariable->iFileName,
+        aVariable->iLine);
+    }
+    return;
+  }
+
+  {
+    int warn = 1;
+    // If a variable is guarded with LocalSymbol it can not interfere with other scripts.
+    if ((*varString)[0] ==  '$') warn = 0;
+    if (aGlobalLazyVariable) warn = 0;
+    if (warn)
+      printf("WARNING: setting global variable \"%s\" (file %s, line %d) (global variables might have undesired side effects, please use Local or LocalSymbols).\n",
+        varString->c_str(),
+        aVariable->iFileName,
+        aVariable->iLine);
+  }
 }
+#endif // YACAS_DEBUG
 
-void LispEnvironment::GetVariable(LispStringPtr aVariable,LispPtr& aResult)
+void LispEnvironment::SetVariable(LispString * aVariable, LispPtr& aValue, LispBoolean aGlobalLazyVariable)
 {
-    aResult.Set(NULL);
-    LispPtr *local = FindLocal(aVariable);
-    if (local != NULL)
-    {
-        aResult.Set(local->Get());
-        return;
-    }
-    LispGlobalVariable *l = iGlobals.LookUp(aVariable);
-    if (l)
-    {
-        if (l->iEvalBeforeReturn)
-        {
-            InternalEval(*this, aResult, l->iValue);
-            l->iValue.Set(aResult.Get());
-            l->iEvalBeforeReturn = LispFalse;
-            return;
-        }
-        else
-        {
-            aResult.Set(l->iValue.Get());
-            return;
-        }
-    }
-}
+  LispPtr *local = FindLocal(aVariable);
+  if (local)
+  {
+    (*local) = (aValue);
+    return;
+  }
 
-void LispEnvironment::SetGlobalEvaluates(LispStringPtr aVariable)
-{
+  iGlobals.SetAssociation(LispGlobalVariable(aValue), aVariable);
+  if (aGlobalLazyVariable)
+  {
+    //TODO we just added the variable! We should not need to re-look it up! Optimize!
     LispGlobalVariable *l = iGlobals.LookUp(aVariable);
-    Check(l != NULL,KLispErrInvalidArg);
+    LISPASSERT(l);
     l->SetEvalBeforeReturn(LispTrue);
+  }
 }
 
-void LispEnvironment::UnsetVariable(LispStringPtr aString)
+void LispEnvironment::GetVariable(LispString * aVariable,LispPtr& aResult)
+{
+  aResult = (NULL);
+  LispPtr *local = FindLocal(aVariable);
+  if (local)
+  {
+    aResult = ((*local));
+    return;
+  }
+  LispGlobalVariable *l = iGlobals.LookUp(aVariable);
+  if (l)
+  {
+    if (l->iEvalBeforeReturn)
+    {
+      InternalEval(*this, aResult, l->iValue);
+      // re-lookup the global variable, as this pointer might now be invalid due to the evaluation actually changing the global itself.
+      l = iGlobals.LookUp(aVariable);
+
+      l->iValue = (aResult);
+      l->iEvalBeforeReturn = LispFalse;
+      return;
+    }
+    else
+    {
+      aResult = (l->iValue);
+      return;
+    }
+  }
+}
+
+void LispEnvironment::UnsetVariable(LispString * aString)
 {
     LispPtr *local = FindLocal(aString);
-    if (local != NULL)
+    if (local)
     {
-        local->Set(NULL);
+        (*local) = (NULL);
         return;
     }
     iGlobals.Release(aString);
@@ -211,15 +253,15 @@ void LispEnvironment::PushLocalFrame(LispBoolean aFenced)
 
 void LispEnvironment::PopLocalFrame()
 {
-    LISPASSERT(iLocalsList != NULL);
+    LISPASSERT(iLocalsList);
     LocalVariableFrame *nextFrame = iLocalsList->iNext;
     delete iLocalsList;
     iLocalsList = nextFrame;
 }
 
-void LispEnvironment::NewLocal(LispStringPtr aVariable,LispObject* aValue)
+void LispEnvironment::NewLocal(LispString * aVariable,LispObject* aValue)
 {
-    LISPASSERT(iLocalsList != NULL);
+    LISPASSERT(iLocalsList);
     iLocalsList->Add(NEW LispLocalVariable(aVariable, aValue));
 }
 
@@ -230,13 +272,13 @@ void LispEnvironment::CurrentLocals(LispPtr& aResult)
 
   LispEnvironment& aEnvironment = *this; //Pity, but we need this for the macros to work
   LispObject* locals = NULL;
-  while (ptr != NULL)
+  while (ptr)
   {
-    locals = LA(ATOML(ptr->iVariable->String()))+LA(locals);
-//    printf("%s ",ptr->iVariable->String());
+    locals = LA(ATOML(ptr->iVariable->c_str()))+LA(locals);
+//    printf("%s ",ptr->iVariable->c_str());
     ptr = ptr->iNext;
   }
-  aResult.Set(LIST(LA(ATOML("List")) + LA(locals)));  
+  aResult = (LIST(LA(ATOML("List")) + LA(locals)));
 }
 
 
@@ -294,9 +336,8 @@ void LispEnvironment::SetCurrentOutput(LispOutput* aOutput)
 LispUserFunction* LispEnvironment::UserFunction(LispPtr& aArguments)
 {
     LispMultiUserFunction* multiUserFunc =
-        iUserFunctions.LookUp(aArguments.Get()->String());
-//    CHECKPTR(multiUserFunc);
-    if (multiUserFunc != NULL)
+        iUserFunctions.LookUp(aArguments->String());
+    if (multiUserFunc)
     {
         LispInt arity = InternalListLength(aArguments)-1;
         CHECKPTR(multiUserFunc->UserFunc(arity));
@@ -306,10 +347,10 @@ LispUserFunction* LispEnvironment::UserFunction(LispPtr& aArguments)
 }
 
 
-LispUserFunction* LispEnvironment::UserFunction(LispStringPtr aName,LispInt aArity)
+LispUserFunction* LispEnvironment::UserFunction(LispString * aName,LispInt aArity)
 {
     LispMultiUserFunction* multiUserFunc = iUserFunctions.LookUp(aName);
-    if (multiUserFunc != NULL)
+    if (multiUserFunc)
     {
         return  multiUserFunc->UserFunc(aArity);
     }
@@ -318,18 +359,18 @@ LispUserFunction* LispEnvironment::UserFunction(LispStringPtr aName,LispInt aAri
 
 
 
-void LispEnvironment::UnFenceRule(LispStringPtr aOperator,LispInt aArity)
+void LispEnvironment::UnFenceRule(LispString * aOperator,LispInt aArity)
 {
     LispMultiUserFunction* multiUserFunc =
         iUserFunctions.LookUp(aOperator);
 
-    Check(multiUserFunc != NULL, KLispErrInvalidArg);
+    Check(multiUserFunc, KLispErrInvalidArg);
     LispUserFunction* userFunc = multiUserFunc->UserFunc(aArity);
-    Check(userFunc != NULL, KLispErrInvalidArg);
+    Check(userFunc, KLispErrInvalidArg);
     userFunc->UnFence();
 }
 
-void LispEnvironment::Retract(LispStringPtr aOperator,LispInt aArity)
+void LispEnvironment::Retract(LispString * aOperator,LispInt aArity)
 {
     LispMultiUserFunction* multiUserFunc = iUserFunctions.LookUp(aOperator);
     if (multiUserFunc)
@@ -338,19 +379,19 @@ void LispEnvironment::Retract(LispStringPtr aOperator,LispInt aArity)
     }
 }
 
-void LispEnvironment::DeclareRuleBase(LispStringPtr aOperator,
+void LispEnvironment::DeclareRuleBase(LispString * aOperator,
                                       LispPtr& aParameters,
                                       LispInt aListed)
 {
     LispMultiUserFunction* multiUserFunc = MultiUserFunction(aOperator);
 
     /*
-     if (multiUserFunc->iFileToOpen != NULL)
+     if (multiUserFunc->iFileToOpen)
     {
         LISPASSERT(multiUserFunc->iFileToOpen->iIsLoaded);
         }
         */
-        
+ 
     // add an operator with this arity to the multiuserfunc.
     BranchingUserFunction *newFunc;
     if (aListed)
@@ -363,15 +404,10 @@ void LispEnvironment::DeclareRuleBase(LispStringPtr aOperator,
     }
     multiUserFunc->DefineRuleBase(newFunc);
 
-#ifdef YACAS_DEBUG
-    {
-        extern long theNrDefinedUser;
-        theNrDefinedUser++;
-    }
-#endif
+  DBG_({ extern long theNrDefinedUser; theNrDefinedUser++; })
 }
 
-void LispEnvironment::DeclareMacroRuleBase(LispStringPtr aOperator, LispPtr& aParameters, LispInt aListed)
+void LispEnvironment::DeclareMacroRuleBase(LispString * aOperator, LispPtr& aParameters, LispInt aListed)
 {
     LispMultiUserFunction* multiUserFunc = MultiUserFunction(aOperator);
     MacroUserFunction *newFunc;
@@ -385,31 +421,26 @@ void LispEnvironment::DeclareMacroRuleBase(LispStringPtr aOperator, LispPtr& aPa
     }
     multiUserFunc->DefineRuleBase(newFunc);
 
-#ifdef YACAS_DEBUG
-    {
-        extern long theNrDefinedUser;
-        theNrDefinedUser++;
-    }
-#endif
+  DBG_({ extern long theNrDefinedUser; theNrDefinedUser++; })
 }
 
 
 
 
-LispMultiUserFunction* LispEnvironment::MultiUserFunction(LispStringPtr aOperator)
+LispMultiUserFunction* LispEnvironment::MultiUserFunction(LispString * aOperator)
 {
     // Find existing multiuser func.
     LispMultiUserFunction* multiUserFunc =
         iUserFunctions.LookUp(aOperator);
 
     // If none exists, add one to the user functions list
-    if (multiUserFunc == NULL)
+    if (!multiUserFunc)
     {
         LispMultiUserFunction newMulti;
         iUserFunctions.SetAssociation(newMulti, aOperator);
         multiUserFunc =
             iUserFunctions.LookUp(aOperator);
-        Check(multiUserFunc != NULL, KLispErrCreatingUserFunction);
+        Check(multiUserFunc, KLispErrCreatingUserFunction);
     }
     return multiUserFunc;
 }
@@ -419,30 +450,30 @@ LispMultiUserFunction* LispEnvironment::MultiUserFunction(LispStringPtr aOperato
 
 
 
-void LispEnvironment::HoldArgument(LispStringPtr  aOperator, LispStringPtr aVariable)
+void LispEnvironment::HoldArgument(LispString *  aOperator, LispString * aVariable)
 {
     LispMultiUserFunction* multiUserFunc =
         iUserFunctions.LookUp(aOperator);
-    Check(multiUserFunc != NULL,KLispErrInvalidArg);
+    Check(multiUserFunc,KLispErrInvalidArg);
     multiUserFunc->HoldArgument(aVariable);
 }
 
 
-void LispEnvironment::DefineRule(LispStringPtr aOperator,LispInt aArity,
+void LispEnvironment::DefineRule(LispString * aOperator,LispInt aArity,
                                  LispInt aPrecedence, LispPtr& aPredicate,
                                  LispPtr& aBody)
 {
     // Find existing multiuser func.
     LispMultiUserFunction* multiUserFunc =
         iUserFunctions.LookUp(aOperator);
-    Check(multiUserFunc != NULL, KLispErrCreatingRule);
+    Check(multiUserFunc, KLispErrCreatingRule);
 
     // Get the specific user function with the right arity
     LispUserFunction* userFunc = multiUserFunc->UserFunc(aArity);
-    Check(userFunc != NULL, KLispErrCreatingRule);
-    
+    Check(userFunc, KLispErrCreatingRule);
+ 
     // Declare a new evaluation rule
-    
+ 
 
     if (IsTrue(*this, aPredicate))
     {
@@ -453,46 +484,49 @@ void LispEnvironment::DefineRule(LispStringPtr aOperator,LispInt aArity,
         userFunc->DeclareRule(aPrecedence, aPredicate,aBody);
 }
 
-void LispEnvironment::DefineRulePattern(LispStringPtr aOperator,LispInt aArity,
+void LispEnvironment::DefineRulePattern(LispString * aOperator,LispInt aArity,
                                         LispInt aPrecedence, LispPtr& aPredicate,
                                         LispPtr& aBody)
 {
     // Find existing multiuser func.
     LispMultiUserFunction* multiUserFunc =
         iUserFunctions.LookUp(aOperator);
-    Check(multiUserFunc != NULL, KLispErrCreatingRule);
+    Check(multiUserFunc, KLispErrCreatingRule);
 
     // Get the specific user function with the right arity
     LispUserFunction* userFunc = multiUserFunc->UserFunc(aArity);
-    Check(userFunc != NULL, KLispErrCreatingRule);
-    
+    Check(userFunc, KLispErrCreatingRule);
+ 
     // Declare a new evaluation rule
     userFunc->DeclarePattern(aPrecedence, aPredicate,aBody);
 }
 
-
-
-void LispEnvironment::SetCommand(YacasEvalCaller aEvaluatorFunc, LispCharPtr aString,LispInt aNrArgs,LispInt aFlags)
+void LispEnvironment::SetCommand(YacasEvalCaller aEvaluatorFunc, LispChar * aString,LispInt aNrArgs,LispInt aFlags)
 {
-#ifdef YACAS_DEBUG
-    extern long theNrDefinedBuiltIn;
-    theNrDefinedBuiltIn++;
-#endif
-    YacasEvaluator eval(aEvaluatorFunc,aNrArgs,aFlags);
-    CoreCommands().SetAssociation(eval,HashTable().LookUp(aString,LispTrue));
+  DBG_({ extern long theNrDefinedBuiltIn; theNrDefinedBuiltIn++; })
+  YacasEvaluator eval(aEvaluatorFunc,aNrArgs,aFlags);
+  /* LispTrue below, because the string is owned externally (meaning the LispString object does not have to free it).
+   * Essentially, the string is either already in the string table, or it is some static data in the executable. Either
+   * way, destruction later on is taken care of.
+   */
+  CoreCommands().SetAssociation(eval,HashTable().LookUp(aString,LispTrue));
 }
 
-void LispEnvironment::RemoveCoreCommand(LispCharPtr aString)
+void LispEnvironment::RemoveCoreCommand(LispChar * aString)
 {
-    CoreCommands().Release(HashTable().LookUp(aString,LispTrue));
+  /* LispTrue below, because the string is owned externally (meaning the LispString object does not have to free it).
+   * Essentially, the string is either already in the string table, or it is some static data in the executable. Either
+   * way, destruction later on is taken care of.
+   */
+  CoreCommands().Release(HashTable().LookUp(aString,LispTrue));
 }
 
-void LispEnvironment::SetUserError(LispCharPtr aErrorString)
+void LispEnvironment::SetUserError(LispChar * aErrorString)
 {
     theUserError=aErrorString;
 }
 
-LispCharPtr LispEnvironment::ErrorString(LispInt aError)
+LispChar * LispEnvironment::ErrorString(LispInt aError)
 {
     LISPASSERT(aError>=0 && aError < KLispNrErrors);
     switch (aError)
@@ -554,8 +588,15 @@ LispCharPtr LispEnvironment::ErrorString(LispInt aError)
     case KLispErrUserInterrupt:
         return "User interrupted calculation";
     case KLispErrUser:
-        if (theUserError != NULL) //TODO should always be true!
+        {
+          // There should be something seriously wrong if this error were raised but no string set for it...
+          LISPASSERT(theUserError != NULL);
+          // Defensive coding, fallthrough
+          if (theUserError)
             return theUserError;
+          else
+            return "Unspecified user error (this should never happen!)";
+        }
         break;
     case KLispErrNonBooleanPredicateInPattern:
         return "Predicate doesn't evaluate to a boolean in pattern";
@@ -568,22 +609,23 @@ LispCharPtr LispEnvironment::ErrorString(LispInt aError)
 
 
 
-LispStringPtr LispEnvironment::FindCachedFile(LispCharPtr aFileName)
+LispString * LispEnvironment::FindCachedFile(LispChar * aFileName)
 {
-    if (iArchive)
+  if (iArchive)
+  {
+    LispInt index = iArchive->iFiles.FindFile(aFileName);
+    if (index>=0)
     {
-        LispInt index = iArchive->iFiles.FindFile(aFileName);
-        if (index>=0)
-        {
-            LispCharPtr contents = iArchive->iFiles.Contents(index);
-            if (contents != NULL)
-            {
-                return NEW LispString(contents,LispFalse);
-            }
-//printf("File %s unsuccessfully decompressed\n",aFileName);
-        }
+      LispChar * contents = iArchive->iFiles.Contents(index);
+      if (contents)
+      {
+        LispString* result = NEW LispString(contents,LispFalse);
+        PlatFree(contents);
+        return result;
+      }
     }
-    return NULL;
+  }
+  return NULL;
 }
 
 
@@ -611,10 +653,9 @@ void LispLocalOutput::Delete()
 
 
 LispLocalEvaluator::LispLocalEvaluator(LispEnvironment& aEnvironment,LispEvaluatorBase* aNewEvaluator)
-: iEnvironment(aEnvironment)
+  : iPreviousEvaluator(aEnvironment.iEvaluator),iEnvironment(aEnvironment)
 {
-    iPreviousEvaluator = aEnvironment.iEvaluator;
-    aEnvironment.iEvaluator = aNewEvaluator;
+  aEnvironment.iEvaluator = aNewEvaluator;
 }
 LispLocalEvaluator::~LispLocalEvaluator()
 {
@@ -622,59 +663,15 @@ LispLocalEvaluator::~LispLocalEvaluator()
     iEnvironment.iEvaluator = iPreviousEvaluator;
 }
 
-LispLocalTrace::LispLocalTrace(LispUserFunction* aUserFunc)
+LispLocalTrace::LispLocalTrace(LispUserFunction* aUserFunc) : iUserFunc(aUserFunc)
 {
-    iUserFunc = aUserFunc;
-    if (iUserFunc!=NULL)
-        iUserFunc->Trace();
+  if (iUserFunc!=NULL)
+    iUserFunc->Trace();
 }
 LispLocalTrace::~LispLocalTrace()
 {
-    if (iUserFunc!=NULL)
-        iUserFunc->UnTrace();
+  if (iUserFunc!=NULL)
+    iUserFunc->UnTrace();
 }
 
 
-
-/*
-void LispErrorNrArgs::PrintError(LispOutput& aOutput)
-{
-    LispChar str[20];
-    aOutput.Write("Expected ");
-    InternalIntToAscii(str,iNrArgsExpected);
-    aOutput.Write(str);
-    aOutput.Write(" arguments, got ");
-    InternalIntToAscii(str,iNrArgsReceived);
-    aOutput.Write(str);
-    aOutput.Write("\n");
-}
-*/
-
-void CDllArray::DeleteNamed(LispCharPtr aName, LispEnvironment& aEnvironment)
-{
-    LispInt i,nr=NrItems();
-    for (i=0;i<nr;i++)
-    {
-//printf("DLL: %s ",(*this)[i]->DllFileName());
-        if (StrEqual(aName,(*this)[i]->DllFileName()))
-        {
-            (*this)[i]->Close(aEnvironment);
-            delete (*this)[i];
-            (*this)[i] = NULL;
-            Delete(i,1);
-//printf("deleted!\n");
-            return;
-        }
-//printf("\n");
-    }
-}
-
-
-
-
-
-
-
-
-
-
