@@ -55,19 +55,15 @@ BranchingUserFunction::BranchPattern::~BranchPattern()
 
 
 BranchingUserFunction::BranchingUserFunction(LispPtr& aParameters)
-    
+  : iParameters(),iRules(),iParamList(aParameters)
 {
-    iParamList.Set(aParameters.Get());
-
-    LispIterator iter(aParameters);
-
-    while (iter())
-    {
-        Check(iter()->String() != NULL,KLispErrCreatingUserFunction);
-        BranchParameter param(iter()->String());
-        iParameters.Append(param);
-        iter.GoNext();
-    }
+  LispIterator iter(aParameters);
+  for ( ; iter.getObj(); ++iter)
+  {
+    Check(iter.getObj()->String(),KLispErrCreatingUserFunction);
+    BranchParameter param(iter.getObj()->String());
+    iParameters.Append(param);
+  }
 }
 
 BranchingUserFunction::~BranchingUserFunction()
@@ -83,75 +79,69 @@ void BranchingUserFunction::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironm
     //hier
     if (Traced())
     {
-        LispPtr tr;
-        tr.Set(LispSubList::New(aArguments.Get()));
+        LispPtr tr(LispSubList::New(aArguments));
         TraceShowEnter(aEnvironment,tr);
-        tr.Set(NULL);
+        tr = (NULL);
     }
 
     LispIterator iter(aArguments);
-    iter.GoNext();
+    ++iter;
 
-    // unrollable arguments 
+    // unrollable arguments
     LispPtr* arguments;
     if (arity==0)
         arguments = NULL;
     else
     {
         LISPASSERT(arity>0);
-        arguments = NEW LispPtr[arity];
+    arguments = NEW LispPtr[arity];
     }
     LocalArgs args(arguments);
 
     // Walk over all arguments, evaluating them as necessary
-    for (i=0;i<arity;i++)
-    {
-        Check(iter() != NULL, KLispErrWrongNumberOfArgs);
+  for (i=0;i<arity;i++,++iter)
+  {
+        Check(iter.getObj(), KLispErrWrongNumberOfArgs);
         if (iParameters[i].iHold)
         {
-            arguments[i].Set(iter()->Copy(LispFalse));
+            arguments[i] = (iter.getObj()->Copy());
         }
         else
         {
-            Check(iter.Ptr() != NULL, KLispErrWrongNumberOfArgs);
-            InternalEval(aEnvironment, arguments[i], *iter.Ptr());
+            //Check(iter.getObj(), KLispErrWrongNumberOfArgs);  // checked above
+            InternalEval(aEnvironment, arguments[i], *iter);
         }
-        iter.GoNext();
-    }
+  }
 
     if (Traced())
     {
         LispIterator iter(aArguments);
-        iter.GoNext();
         for (i=0;i<arity;i++)
         {
-            TraceShowArg(aEnvironment,*iter.Ptr(),
-                  arguments[i]);
-
-            iter.GoNext();
+            TraceShowArg(aEnvironment,*++iter,arguments[i]);
         }
-    }
-    
+  }
+ 
     // declare a new local stack.
     LispLocalFrame frame(aEnvironment,Fenced());
 
     // define the local variables.
     for (i=0;i<arity;i++)
     {
-        LispStringPtr variable = iParameters[i].iParameter;
+        LispString * variable = iParameters[i].iParameter;
         // set the variable to the new value
-        aEnvironment.NewLocal(variable,arguments[i].Get());
+        aEnvironment.NewLocal(variable,arguments[i]);
     }
 
     // walk the rules database, returning the evaluated result if the
     // predicate is LispTrue.
-    LispInt nrRules = iRules.NrItems();
+    LispInt nrRules = iRules.Size();
     UserStackInformation &st = aEnvironment.iEvaluator->StackInformation();
     for (i=0;i<nrRules;i++)
     {
         BranchRuleBase* thisRule = iRules[i];
         CHECKPTR(thisRule);
-        LISPASSERT(thisRule != NULL);
+        LISPASSERT(thisRule);
 
         st.iRulePrecedence = thisRule->Precedence();
         LispBoolean matches = thisRule->Matches(aEnvironment, arguments);
@@ -165,42 +155,40 @@ void BranchingUserFunction::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironm
         // If rules got inserted, walk back
         while (thisRule != iRules[i] && i>0) i--;
     }
-    
+ 
     // No predicate was LispTrue: return a new expression with the evaluated
     // arguments.
 
     {
-        LispPtr full;
-        full.Set(aArguments.Get()->Copy(LispFalse));
+        LispPtr full(aArguments->Copy());
         if (arity == 0)
         {
-            full.Get()->Next().Set(NULL);
+            full->Nixed() = (NULL);
         }
         else
         {
-            full.Get()->Next().Set(arguments[0].Get());
+            full->Nixed() = (arguments[0]);
             for (i=0;i<arity-1;i++)
             {
-                arguments[i].Get()->Next().Set(arguments[i+1].Get());
+                arguments[i]->Nixed() = (arguments[i+1]);
             }
         }
-        aResult.Set(LispSubList::New(full.Get()));
+        aResult = (LispSubList::New(full));
     }
 
 FINISH:
     if (Traced())
     {
-        LispPtr tr;
-        tr.Set(LispSubList::New(aArguments.Get()));
+        LispPtr tr(LispSubList::New(aArguments));
         TraceShowLeave(aEnvironment, aResult,tr);
-        tr.Set(NULL);
+        tr = (NULL);
     }
 }
 
-void BranchingUserFunction::HoldArgument(LispStringPtr aVariable)
+void BranchingUserFunction::HoldArgument(LispString * aVariable)
 {
     LispInt i;
-    LispInt nrc=iParameters.NrItems();
+    LispInt nrc=iParameters.Size();
     for (i=0;i<nrc;i++)
     {
         if (iParameters[i].iParameter == aVariable)
@@ -210,7 +198,7 @@ void BranchingUserFunction::HoldArgument(LispStringPtr aVariable)
 
 LispInt BranchingUserFunction::Arity() const
 {
-    return iParameters.NrItems();
+    return iParameters.Size();
 }
 
 LispInt BranchingUserFunction::IsArity(LispInt aArity) const
@@ -223,15 +211,16 @@ void BranchingUserFunction::DeclareRule(LispInt aPrecedence, LispPtr& aPredicate
 {
     // New branching rule.
     BranchRule* newRule = NEW BranchRule(aPrecedence,aPredicate,aBody);
-    Check(newRule != NULL,KLispErrCreatingRule);
+    Check(newRule,KLispErrCreatingRule);
 
     InsertRule(aPrecedence,newRule);
 }
- void BranchingUserFunction::DeclareRule(LispInt aPrecedence, LispPtr& aBody)
+
+void BranchingUserFunction::DeclareRule(LispInt aPrecedence, LispPtr& aBody)
 {
     // New branching rule.
     BranchRule* newRule = NEW BranchRuleTruePredicate(aPrecedence,aBody);
-    Check(newRule != NULL,KLispErrCreatingRule);
+    Check(newRule,KLispErrCreatingRule);
 
     InsertRule(aPrecedence,newRule);
 }
@@ -241,18 +230,17 @@ void BranchingUserFunction::DeclarePattern(LispInt aPrecedence, LispPtr& aPredic
 {
     // New branching rule.
     BranchPattern* newRule = NEW BranchPattern(aPrecedence,aPredicate,aBody);
-    Check(newRule != NULL,KLispErrCreatingRule);
+    Check(newRule,KLispErrCreatingRule);
 
     InsertRule(aPrecedence,newRule);
 }
 
-
 void BranchingUserFunction::InsertRule(LispInt aPrecedence,BranchRuleBase* newRule)
 {
     // Find place to insert
     LispInt low,high,mid;
     low=0;
-    high=iRules.NrItems();
+    high=iRules.Size();
 
     // Constant time: find out if the precedence is before any of the
     // currently defined rules or past them.
@@ -298,14 +286,11 @@ void BranchingUserFunction::InsertRule(LispInt aPrecedence,BranchRuleBase* newRu
     iRules.Insert(mid,newRule);
 }
 
-
 LispPtr& BranchingUserFunction::ArgList()
 {
     return iParamList;
 }
 
-
-
 ListedBranchingUserFunction::ListedBranchingUserFunction(LispPtr& aParameters)
     : BranchingUserFunction(aParameters)
 {
@@ -321,226 +306,208 @@ LispInt ListedBranchingUserFunction::IsArity(LispInt aArity) const
 void ListedBranchingUserFunction::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,
                                            LispPtr& aArguments)
 {
-    LispPtr newArgs;
-    LispIterator iter(aArguments);
-    LispPtr* ptr =  &newArgs;
-    LispInt arity = Arity();
-    LispInt i=0;
-    while (i < arity && iter() != NULL)
-    {
-        ptr->Set(iter()->Copy(LispFalse));
-        ptr = &(ptr->Get()->Next());
-        i++;
-        iter.GoNext();
-    }
-    if (iter()->Next().Get() == NULL)
-    {
-        ptr->Set(iter()->Copy(LispFalse));
-        ptr = &(ptr->Get()->Next());
-        i++;
-        iter.GoNext();
-        LISPASSERT(iter() == NULL);
-    }
-    else
-    {
-        LispPtr head;
-        head.Set(aEnvironment.iList->Copy(LispFalse));
-        head.Get()->Next().Set(iter());
-        ptr->Set(LispSubList::New(head.Get()));
-    }
-    BranchingUserFunction::Evaluate(aResult, aEnvironment, newArgs);
+  LispPtr newArgs;
+  LispIterator iter(aArguments);
+  LispPtr* ptr =  &newArgs;
+  LispInt arity = Arity();
+  // Make a copy of the arguments first
+  // TODO: if we were to change the internal representation to a cons cell, this copying would not be needed
+  LispInt i;
+  for (i = 0; i < arity && iter.getObj(); i++,++iter)
+  {
+    (*ptr) = (iter.getObj()->Copy());
+    ptr = &((*ptr)->Nixed());
+  }
+  if (!iter.getObj()->Nixed())
+  {
+    (*ptr) = (iter.getObj()->Copy());
+    ptr = &((*ptr)->Nixed());
+    ++iter;
+    LISPASSERT(!iter.getObj());
+  }
+  else
+  {
+    LispPtr head(aEnvironment.iList->Copy());
+    head->Nixed() = (iter.getObj());
+    (*ptr) = (LispSubList::New(head));
+  }
+  BranchingUserFunction::Evaluate(aResult, aEnvironment, newArgs);
 }
 
-
-
 MacroUserFunction::MacroUserFunction(LispPtr& aParameters)
   : BranchingUserFunction(aParameters)
 {
     LispIterator iter(aParameters);
-    LispInt i=0;
-    while (iter())
+    for (LispInt i=0; iter.getObj(); i++,++iter)
     {
-        Check(iter()->String() != NULL,KLispErrCreatingUserFunction);
+        Check(iter.getObj()->String(),KLispErrCreatingUserFunction);
         iParameters[i].iHold = LispTrue;
-        iter.GoNext();
-        i++;
     }
-    UnFence();
+  UnFence();
 }
 
 void MacroUserFunction::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,
               LispPtr& aArguments)
 {
-    LispInt arity = Arity();
-    LispInt i;
+  LispInt arity = Arity();
+  LispInt i;
 
-    //hier
-    if (Traced())
+  if (Traced())
+  {
+    LispPtr tr(LispSubList::New(aArguments));
+    TraceShowEnter(aEnvironment,tr);
+    tr = (NULL);
+  }
+
+  LispIterator iter(aArguments);
+  ++iter;
+
+  // unrollable arguments
+  LispPtr* arguments;
+  if (arity==0)
+    arguments = NULL;
+  else
+  {
+    LISPASSERT(arity>0);
+    arguments = NEW LispPtr[arity];
+  }
+  LocalArgs args(arguments);
+
+  // Walk over all arguments, evaluating them as necessary
+  for (i=0;i<arity;i++,++iter)
+  {
+    Check(iter.getObj(), KLispErrWrongNumberOfArgs);
+    if (iParameters[i].iHold)
     {
-        LispPtr tr;
-        tr.Set(LispSubList::New(aArguments.Get()));
-        TraceShowEnter(aEnvironment,tr);
-        tr.Set(NULL);
+      arguments[i] = (iter.getObj()->Copy());
     }
-
-    LispIterator iter(aArguments);
-    iter.GoNext();
-
-    // unrollable arguments 
-    LispPtr* arguments;
-    if (arity==0)
-        arguments = NULL;
     else
     {
-        LISPASSERT(arity>0);
-        arguments = NEW LispPtr[arity];
+      InternalEval(aEnvironment, arguments[i], *iter);
     }
-    LocalArgs args(arguments);
+  }
 
-    // Walk over all arguments, evaluating them as necessary
+  if (Traced())
+  {
+    LispIterator iter(aArguments);
+    // TODO: ideally we would only need an iterator here
+    ++iter;
     for (i=0;i<arity;i++)
     {
-        Check(iter() != NULL, KLispErrWrongNumberOfArgs);
-        if (iParameters[i].iHold)
-        {
-            arguments[i].Set(iter()->Copy(LispFalse));
-        }
-        else
-        {
-            Check(iter.Ptr() != NULL, KLispErrWrongNumberOfArgs);
-            InternalEval(aEnvironment, arguments[i], *iter.Ptr());
-        }
-        iter.GoNext();
+      TraceShowArg(aEnvironment,*iter,arguments[i]);
+      ++iter;
+    }
+  }
+
+  LispPtr substedBody;
+  {
+    // declare a new local stack.
+    LispLocalFrame frame(aEnvironment,LispFalse);
+
+    // define the local variables.
+    for (i=0;i<arity;i++)
+    {
+      LispString * variable = iParameters[i].iParameter;
+      // set the variable to the new value
+      aEnvironment.NewLocal(variable,arguments[i]);
     }
 
-    if (Traced())
+    // walk the rules database, returning the evaluated result if the
+    // predicate is LispTrue.
+    LispInt nrRules = iRules.Size();
+    UserStackInformation &st = aEnvironment.iEvaluator->StackInformation();
+    for (i=0;i<nrRules;i++)
     {
-        LispIterator iter(aArguments);
-        iter.GoNext();
-        for (i=0;i<arity;i++)
-        {
-            TraceShowArg(aEnvironment,*iter.Ptr(),
-                  arguments[i]);
+      BranchRuleBase* thisRule = iRules[i];
+      CHECKPTR(thisRule);
+      LISPASSERT(thisRule);
 
-            iter.GoNext();
-        }
-    }
-
-    LispPtr substedBody;
-    {
-      // declare a new local stack.
-      LispLocalFrame frame(aEnvironment,LispFalse);
-  
-      // define the local variables.
-      for (i=0;i<arity;i++)
+      st.iRulePrecedence = thisRule->Precedence();
+      LispBoolean matches = thisRule->Matches(aEnvironment, arguments);
+      if (matches)
       {
-          LispStringPtr variable = iParameters[i].iParameter;
-          // set the variable to the new value
-          aEnvironment.NewLocal(variable,arguments[i].Get());
-      }
-  
-      // walk the rules database, returning the evaluated result if the
-      // predicate is LispTrue.
-      LispInt nrRules = iRules.NrItems();
-      UserStackInformation &st = aEnvironment.iEvaluator->StackInformation();
-      for (i=0;i<nrRules;i++)
-      {
-          BranchRuleBase* thisRule = iRules[i];
-          CHECKPTR(thisRule);
-          LISPASSERT(thisRule != NULL);
-  
-          st.iRulePrecedence = thisRule->Precedence();
-          LispBoolean matches = thisRule->Matches(aEnvironment, arguments);
-          if (matches)
-          {
-              st.iSide = 1;
+        st.iSide = 1;
 
-              BackQuoteBehaviour behaviour(aEnvironment);
-              InternalSubstitute(substedBody, thisRule->Body(), behaviour);
-//              InternalEval(aEnvironment, aResult, thisRule->Body());
-              break;
-          }
-  
-          // If rules got inserted, walk back
-          while (thisRule != iRules[i] && i>0) i--;
+        BackQuoteBehaviour behaviour(aEnvironment);
+        InternalSubstitute(substedBody, thisRule->Body(), behaviour);
+        break;
       }
+ 
+      // If rules got inserted, walk back
+      while (thisRule != iRules[i] && i>0) i--;
     }
-      
+  }
 
-    if (substedBody.Get())
+  if (!!substedBody)
+  {
+    InternalEval(aEnvironment, aResult, substedBody);
+  }
+  else
+  // No predicate was LispTrue: return a new expression with the evaluated
+  // arguments.
+  {
+    LispPtr full(aArguments->Copy());
+    if (arity == 0)
     {
-        InternalEval(aEnvironment, aResult, substedBody);
+      full->Nixed() = (NULL);
     }
     else
-    // No predicate was LispTrue: return a new expression with the evaluated
-    // arguments.
-      {
-        LispPtr full;
-        full.Set(aArguments.Get()->Copy(LispFalse));
-        if (arity == 0)
-        {
-            full.Get()->Next().Set(NULL);
-        }
-        else
-        {
-            full.Get()->Next().Set(arguments[0].Get());
-            for (i=0;i<arity-1;i++)
-            {
-                arguments[i].Get()->Next().Set(arguments[i+1].Get());
-            }
-        }
-        aResult.Set(LispSubList::New(full.Get()));
-    }
-//FINISH:
-    if (Traced())
     {
-        LispPtr tr;
-        tr.Set(LispSubList::New(aArguments.Get()));
-        TraceShowLeave(aEnvironment, aResult,tr);
-        tr.Set(NULL);
+      full->Nixed() = (arguments[0]);
+      for (i=0;i<arity-1;i++)
+      {
+        arguments[i]->Nixed() = (arguments[i+1]);
+      }
     }
+    aResult = (LispSubList::New(full));
+  }
+  if (Traced())
+  {
+    LispPtr tr(LispSubList::New(aArguments));
+    TraceShowLeave(aEnvironment, aResult,tr);
+    tr = (NULL);
+  }
 }
 
-
 ListedMacroUserFunction::ListedMacroUserFunction(LispPtr& aParameters)
   : MacroUserFunction(aParameters)
 {
 }
+
 LispInt ListedMacroUserFunction::IsArity(LispInt aArity) const
 {
     return (Arity() <= aArity);
 }
+
 void ListedMacroUserFunction::Evaluate(LispPtr& aResult,LispEnvironment& aEnvironment,
               LispPtr& aArguments)
 {
-    LispPtr newArgs;
-    LispIterator iter(aArguments);
-    LispPtr* ptr =  &newArgs;
-    LispInt arity = Arity();
-    LispInt i=0;
-    while (i < arity && iter() != NULL)
-    {
-        ptr->Set(iter()->Copy(LispFalse));
-        ptr = &(ptr->Get()->Next());
-        i++;
-        iter.GoNext();
-    }
-    if (iter()->Next().Get() == NULL)
-    {
-        ptr->Set(iter()->Copy(LispFalse));
-        ptr = &(ptr->Get()->Next());
-        i++;
-        iter.GoNext();
-        LISPASSERT(iter() == NULL);
-    }
-    else
-    {
-        LispPtr head;
-        head.Set(aEnvironment.iList->Copy(LispFalse));
-        head.Get()->Next().Set(iter());
-        ptr->Set(LispSubList::New(head.Get()));
-    }
-    MacroUserFunction::Evaluate(aResult, aEnvironment, newArgs);
+  LispPtr newArgs;
+  LispIterator iter(aArguments);
+  LispPtr* ptr =  &newArgs;
+  LispInt arity = Arity();
+  LispInt i=0;
+  // TODO: the code would look a lot easier if we could do with only an iterator
+  while (i < arity && iter.getObj())
+  {
+    (*ptr) = (iter.getObj()->Copy());
+    ptr = &((*ptr)->Nixed());
+    i++;
+    ++iter;
+  }
+  if (!iter.getObj()->Nixed())
+  {
+    (*ptr) = (iter.getObj()->Copy());
+    ptr = &((*ptr)->Nixed());
+    i++;
+    ++iter;
+    LISPASSERT(!iter.getObj());
+  }
+  else
+  {
+    LispPtr head(aEnvironment.iList->Copy());
+    head->Nixed() = (iter.getObj());
+    (*ptr) = (LispSubList::New(head));
+  }
+  MacroUserFunction::Evaluate(aResult, aEnvironment, newArgs);
 }
-
-
