@@ -14,7 +14,7 @@ void CStdCommandLine::Pause()
 {
 }
 
-void CStdCommandLine::ShowLine(LispCharPtr prompt,LispInt promptlen,LispInt cursor)
+void CStdCommandLine::ShowLine(LispChar * prompt,LispInt promptlen,LispInt cursor)
 {
 }
 
@@ -31,13 +31,14 @@ LispInt CStdCommandLine::GetKey()
     return '\n';
 }
 
-void CStdCommandLine::ReadLine(LispCharPtr prompt)
+void CStdCommandLine::ReadLine(LispChar * prompt)
 {
     printf(prompt); fflush(stdout);
     char buffer[4001];
     int offs=0;
 MORE:
     fgets(&buffer[offs],4000-offs,stdin);
+if (feof(stdin)) strcpy(buffer,"quit");
     offs=strlen(buffer);
 
     if (buffer[offs-1] == '\n')
@@ -50,8 +51,8 @@ MORE:
         goto MORE;
     if (buffer[offs-1] == '\\')
         goto MORE;
-    iLine.GrowTo(offs+1);
-    iLine.SetNrItems(offs+1);
+    iLine.ResizeTo(offs+1);
+    //iLine.Resize(offs+1);
     strcpy(&iLine[0],buffer);
 }
 
