@@ -36,53 +36,51 @@ InputStatus::~InputStatus()
 
 
 StdFileInput::StdFileInput(FILE* aFile,InputStatus& aStatus)
-    : LispInput(aStatus)
+    : LispInput(aStatus),  iFile(aFile)
 {
-    iFile = aFile;
 }
 StdFileInput::StdFileInput(LispLocalFile& aFile,InputStatus& aStatus)
-    : LispInput(aStatus)
+    : LispInput(aStatus),iFile(aFile.iFile)
 {
-    iFile = aFile.iFile;
 }
 
 
 
 LispChar StdFileInput::Next()
 {
-    LispChar c;
-    c=fgetc(iFile);
-    if (c == '\n')
-        iStatus.NextLine();
-    return c;
+  LispChar c;
+  c=fgetc(iFile);
+  if (c == '\n')
+    iStatus.NextLine();
+  return c;
 }
 
-LispChar StdFileInput::Peek() 
+LispChar StdFileInput::Peek()
 {
-    LispChar c = fgetc(iFile);
-    ungetc(c,iFile);
-    return c;
+  LispChar c = fgetc(iFile);
+  ungetc(c,iFile);
+  return c;
 }
  
 void StdFileInput::Rewind()
 {
-    fseek(iFile,0,SEEK_SET);
+  fseek(iFile,0,SEEK_SET);
 }
 
-LispBoolean StdFileInput::EndOfStream() 
+LispBoolean StdFileInput::EndOfStream()
 {
-    return feof(iFile);
+  return feof(iFile);
 }
 
-LispCharPtr StdFileInput::StartPtr()
+LispChar * StdFileInput::StartPtr()
 {
-    LISPASSERT(0);
-    return NULL;
+  LISPASSERT(0);
+  return NULL;
 }
 LispInt StdFileInput::Position()
 {
     LISPASSERT(0);
-    return 0;
+  return 0;
 }
 
 void StdFileInput::SetPosition(LispInt aPosition)
@@ -113,18 +111,18 @@ CachedStdFileInput::~CachedStdFileInput()
     PlatFree(iBuffer);
 }
 
-CachedStdFileInput::CachedStdFileInput(LispLocalFile& aFile,InputStatus& aStatus) : StdFileInput(aFile,aStatus)
+CachedStdFileInput::CachedStdFileInput(LispLocalFile& aFile,InputStatus& aStatus) : StdFileInput(aFile,aStatus),iBuffer(NULL),iCurrentPos(0),iNrBytes(0)
 {
-    // Get size of file
-    fseek(iFile,0,SEEK_END);
-    iNrBytes = ftell(iFile);
-    fseek(iFile,0,SEEK_SET);
-    // Read in the full buffer
-    iBuffer = PlatAlloc(iNrBytes+1);
-    Check(iBuffer!=NULL,KLispErrNotEnoughMemory);
-    iCurrentPos = 0;
-    fread(iBuffer,iNrBytes,1,iFile);
-    iBuffer[iNrBytes] = '\0';
+  // Get size of file
+  fseek(iFile,0,SEEK_END);
+  iNrBytes = ftell(iFile);
+  fseek(iFile,0,SEEK_SET);
+  // Read in the full buffer
+  char * ptr = PlatAllocN<char>(iNrBytes+1);      // sizeof(char) == 1
+  iBuffer = (LispChar *)ptr;                                        // sizeof(LispChar) == not so sure
+  Check(ptr!=NULL,KLispErrNotEnoughMemory);
+  fread(ptr,iNrBytes,1,iFile);
+  ptr[iNrBytes] = '\0';
 }
 
 LispChar CachedStdFileInput::Next()
@@ -149,7 +147,7 @@ LispChar CachedStdFileInput::Peek()
 
 void CachedStdFileInput::Rewind()
 {
-	iCurrentPos = 0;
+  iCurrentPos = 0;
 }
 
 LispBoolean CachedStdFileInput::EndOfStream()
@@ -157,7 +155,7 @@ LispBoolean CachedStdFileInput::EndOfStream()
     return (iCurrentPos >= iNrBytes);
 }
 
-LispCharPtr CachedStdFileInput::StartPtr()
+LispChar * CachedStdFileInput::StartPtr()
 {
     return iBuffer;
 }
@@ -173,8 +171,9 @@ void CachedStdFileInput::SetPosition(LispInt aPosition)
 }
 
 
-void InternalFindFile(LispCharPtr aFileName, InputDirectories& aInputDirectories,
-                     LispCharPtr aFoundFile)
+// TODO: woof -- buffer overflow problems in here?
+void InternalFindFile(LispChar * aFileName, InputDirectories& aInputDirectories,
+                     LispChar * aFoundFile)
 {
     strcpy(aFoundFile,aFileName);
 
@@ -182,15 +181,15 @@ void InternalFindFile(LispCharPtr aFileName, InputDirectories& aInputDirectories
 
     FILE* file = fopen(aFoundFile,"rb");
     LispInt i=0;
-    while (file == NULL && i<aInputDirectories.NrItems())
+    while (!file && i<aInputDirectories.Size())
     {
-        strcpy(aFoundFile,aInputDirectories[i]->String());
+        strcpy(aFoundFile,aInputDirectories[i]->c_str());
         strcat(aFoundFile,aFileName);
         MapPathSeparators(aFoundFile);
         file = fopen(aFoundFile,"rb");
         i++;
     }
-    if (file != NULL)
+    if (file)
     {
         fclose(file);
     }
@@ -201,106 +200,104 @@ void InternalFindFile(LispCharPtr aFileName, InputDirectories& aInputDirectories
 }
 
 LispLocalFile::LispLocalFile(LispEnvironment& aEnvironment,
-                             LispCharPtr aFileName, LispBoolean aRead,
+                             LispChar * aFileName, LispBoolean aRead,
                              InputDirectories& aInputDirectories)
-: iEnvironment(aEnvironment)
+  : iFile(NULL),iEnvironment(aEnvironment),iOpened(LispFalse)
 {
-    LispChar othername[1024];//TODO
-    if (aRead)
-    {
-        strcpy(othername,aFileName);
-        MapPathSeparators(othername);
+  LispChar othername[1024];//TODO
+  if (aRead)
+  {
+    strcpy(othername,aFileName);
+    MapPathSeparators(othername);
 
-        iFile = fopen(othername,"rb");
-        LispInt i=0;
-        while (iFile == NULL && i<aInputDirectories.NrItems())
-        {
-            strcpy(othername,aInputDirectories[i]->String());
-            strcat(othername,aFileName);
-            MapPathSeparators(othername);
-            iFile = fopen(othername,"rb");
-            i++;
-        }
-    }
-    else
+    iFile = fopen(othername,"rb");
+    LispInt i=0;
+    while (!iFile && i<aInputDirectories.Size())
     {
-      strcpy(othername,aFileName);
+      strcpy(othername,aInputDirectories[i]->c_str());
+      strcat(othername,aFileName);
       MapPathSeparators(othername);
-      iFile = fopen(othername,"w");
+      iFile = fopen(othername,"rb");
+      i++;
     }
+  }
+  else
+  {
+    strcpy(othername,aFileName);
+    MapPathSeparators(othername);
+    iFile = fopen(othername,"w");
+  }
 
-    if (iFile == NULL)
-        iOpened=0;
-    else
-        iOpened=1;
+  if (!iFile)
+    iOpened=0;
+  else
+    iOpened=1;
 
-    SAFEPUSH(iEnvironment,*this);
+  SAFEPUSH(iEnvironment,*this);
 }
 
 //aRead is for opening in read mode (otherwise opened in write mode)
 LispLocalFile::~LispLocalFile()
 {
-    SAFEPOP(iEnvironment);
-    Delete();
+  SAFEPOP(iEnvironment);
+  Delete();
 }
 
 void LispLocalFile::Delete()
 {
-    if (iFile)
-        fclose(iFile);
-    iFile = NULL;
+  if (iFile)
+    fclose(iFile);
+  iFile = NULL;
 }
 
 
 
 
 CachedStdUserInput::CachedStdUserInput(InputStatus& aStatus) :
-StdUserInput(aStatus)
+StdUserInput(aStatus),iBuffer(),iCurrentPos(0)
 {
-//printf("CachedStdUserInput:construct\n");
-    Rewind();
+  Rewind();
 }
 LispChar CachedStdUserInput::Next()
 {
-//printf("CachedStdUserInput:Next\n");
-    LispChar c = Peek();
-    iCurrentPos++;
-    printf("%c",c);
-    return c;
+  LispChar c = Peek();
+  iCurrentPos++;
+  printf("%c",c);
+  return c;
 }
 
 LispChar CachedStdUserInput::Peek()
 {
-    if (iCurrentPos == iBuffer.NrItems())
-    {
-        iBuffer.Append(fgetc(iFile));
-    }
-    return iBuffer[iCurrentPos];
+  if (iCurrentPos == iBuffer.Size())
+  {
+    iBuffer.Append(fgetc(iFile));
+  }
+  return iBuffer[iCurrentPos];
 }
 
 LispBoolean CachedStdUserInput::EndOfStream()
 {
-    return LispFalse;
+  return LispFalse;
 }
 
 void CachedStdUserInput::Rewind()
 {
-    // Make sure there is a buffer to point to.
-    iBuffer.GrowTo(10);
-    iBuffer.SetNrItems(0);
-    iCurrentPos=0;
+  // Make sure there is a buffer to point to.
+  iBuffer.ResizeTo(10);
+  iBuffer.ResizeTo(0);
+  iCurrentPos=0;
 }
 
-LispCharPtr CachedStdUserInput::StartPtr()
+LispChar * CachedStdUserInput::StartPtr()
 {
-    if (iBuffer.NrItems() == 0)
-        Peek();
-    return &iBuffer[0];
+  if (iBuffer.Size() == 0)
+    Peek();
+  return &iBuffer[0];
 }
 
 LispInt CachedStdUserInput::Position()
 {
-    return iCurrentPos;
+  return iCurrentPos;
 }
 
 
