@@ -25,6 +25,8 @@
 //      - c : inhibits printing the prompt to the console
 //      - t : follow history when entering on the command line
 //      - w : hides the console window in Windows
+//   4)
+//  -i <command> : execute <command>
 //
 // Example: 'yacas -pc' will use minimal command line interaction,
 //          showing no prompts, and with no readline functionality.
@@ -42,33 +44,21 @@
 #include <signal.h>
 #endif
 
-/*This is not the right place to define path separators. Better to define "/" as the Yacas path separator,
-  and then to handle the actual mapping at the lowest level.
-  TODO remove as soon as I get this working under Windows
-#ifdef WIN32
-  #define PATH_SEPARATOR   '\\' 
-  #define PATH_SEPARATOR_2 "\\\\" // Yes, four! Escaped once by C, then in Yacas.
-#else
-*/
-  #define PATH_SEPARATOR   '/'
-  #define PATH_SEPARATOR_2 "/"
-//#endif
+// For all platforms, assume forward slash as path separator (handle at the lowest level).
+#define PATH_SEPARATOR   '/'
+#define PATH_SEPARATOR_2 "/"
 
-#include "plugins_available.h"
 #include "yacas.h"
 
 #ifndef WIN32
   #include "unixcommandline.h"
   #define FANCY_COMMAND_LINE CUnixCommandLine
-  /* PLATFORM_OS is defined in config.h */
-#else  
-  #define _WINSOCKAPI_            // Prevent inclusion of winsock.h in windows.h 
-  #define _WIN32_WINDOWS 0x0410      // Make sure that Waitable Timer functions are declared in winbase.h 
-  #include "win32commandline.h"      
+#else
+  #define _WINSOCKAPI_            // Prevent inclusion of winsock.h in windows.h
+  #define _WIN32_WINDOWS 0x0410      // Make sure that Waitable Timer functions are declared in winbase.h
+  #include "win32commandline.h"
   #define FANCY_COMMAND_LINE CWin32CommandLine
   #define SCRIPT_DIR ""
-  #define PLUGIN_DIR ""
-  #define PLATFORM_OS "Win32"
 #endif
 
 #include "stdcommandline.h"
@@ -77,12 +67,10 @@
 #include "arggetter.h"
 #include "archiver.h"
 
-//TODO time.h and errors.h are only needed for LispTime
+//time.h and errors.h are only needed for LispTime
 #include <time.h>
 #include "errors.h"
 
-//#include "anumber.h"
-
 
 #ifndef VERSION
 //#define VERSION "Windows latest"
@@ -118,14 +106,13 @@ CCommandLine *commandline = NULL;
 int use_stdin = 0;
 int use_plain = 0;
 int show_prompt = 1;
-int trace_history = 0;
 int use_texmacs_out = 0;
 int compiled_plugins = 1;
 
 int stack_size = 50000;
 
 #ifdef YACAS_DEBUG
-  int verbose_debug=0;
+int verbose_debug=0;
 #endif
 int patchload=0;
 int winsockinitialised=0;
@@ -134,16 +121,8 @@ int hideconsolewindow=0;
 #ifndef SCRIPT_DIR
   #define SCRIPT_DIR "none"
 #endif
-#ifndef PLUGIN_DIR
-  #define PLUGIN_DIR "none"
-#endif
-#ifndef PLATFORM_OS
-  #define PLATFORM_OS "none"
-#endif
-
 
 char* root_dir    = SCRIPT_DIR;
-char* dll_dir     = PLUGIN_DIR;
 #ifndef WIN32
   char* archive     = NULL;
 #else
@@ -160,11 +139,11 @@ static int readmode = 0;
 int compressed_archive = 1;
 
 int server_mode = 0;
+int server_single_user = 0;
 int server_port = 9734;
 
 char* execute_commnd = NULL;
 
-//TODO global!!!
 static LispBoolean busy=LispTrue;
 static LispBoolean restart=LispFalse;
 
@@ -174,7 +153,7 @@ static LispBoolean Busy()
 }
 
 
-  
+ 
 void ReportNrCurrent()
 {
 #ifdef YACAS_DEBUG
@@ -202,10 +181,6 @@ void ReportNrCurrent()
 
 #define RESULT aEnvironment.iStack.GetElement(aStackTop)
 #define ARGUMENT(i) aEnvironment.iStack.GetElement(aStackTop+i)
-void LispPlatformOS(LispEnvironment& aEnvironment, LispInt aStackTop)
-{
-  RESULT.Set(LispAtom::New(aEnvironment,"\"" PLATFORM_OS "\""));
-}
 
 void LispExit(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
@@ -213,22 +188,6 @@ void LispExit(LispEnvironment& aEnvironment, LispInt aStackTop)
     InternalTrue(aEnvironment, RESULT);
 }
 
-void LispPluginsCanBeLoaded(LispEnvironment& aEnvironment, LispInt aStackTop)
-{
-#ifndef DISABLE_DYNAMIC 
-  InternalTrue(aEnvironment, RESULT);
-#else // DISABLE_DYNAMIC 
-
-#ifdef EXE_DLL_PLUGINS
-  InternalTrue(aEnvironment, RESULT);
-#else // EXE_DLL_PLUGINS
-  InternalFalse(aEnvironment, RESULT);
-#endif // EXE_DLL_PLUGINS
-
-#endif // DISABLE_DYNAMIC 
-}
-
-
 
 void LispExitRequested(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
@@ -245,7 +204,7 @@ void LispStackSize(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
     LispChar buf[30];
     InternalIntToAscii(buf, (int)(the_first_stack_var-(unsigned char*)&buf[0]));
-    RESULT.Set(LispAtom::New(aEnvironment,buf));
+    RESULT = (LispAtom::New(aEnvironment,buf));
 }
 
 
@@ -253,49 +212,23 @@ char* ReadInputString(char* prompt)
 {
   if (!commandline) return "False";
   char *inpline;
-REDO:
   readmode = 1;
   commandline->ReadLine(prompt);
   readmode = 0;
-  inpline =  commandline->iLine.String();
+  inpline =  commandline->iLine.c_str();
 
   if (inpline)
   {
     if(*inpline)
     {
-      if (inpline[0] == '?')
+      if (!strncmp(inpline,"restart",7))
       {
-        if (inpline[1] == '?')
-        {
-          yacas->Evaluate("Help()");
-          goto REDO;
-        }
-        else
-        {
-          if (strlen(&inpline[1]) < 100)
-          {
-            char buf[120];
-#ifdef HAVE_VSNPRINTF
-            snprintf(buf,120,"Help(\"%s\")",&inpline[1]);
-#else
-            sprintf(buf,"Help(\"%s\")",&inpline[1]);
-#endif
-            yacas->Evaluate(buf);
-            goto REDO;
-          }
-        }
+        restart=LispTrue;
+        busy=LispFalse;
       }
-      else
+      else if (!strncmp(inpline,"quit",4))
       {
-        if (!strncmp(inpline,"restart",7))
-        {
-          restart=LispTrue;
-          busy=LispFalse;
-        }
-        else if (!strncmp(inpline,"quit",4))
-        {
-          busy=LispFalse;
-        }
+        busy=LispFalse;
       }
     }
   }
@@ -307,13 +240,12 @@ REDO:
 
 static void LispReadCmdLineString(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    LispPtr promptObject;
-    promptObject.Set(ARGUMENT(1).Get());
+    LispPtr promptObject = (ARGUMENT(1));
     CHK_ISSTRING_CORE(promptObject,1);
     LispString prompt;
-    InternalUnstringify(prompt, promptObject.Get()->String());
-    char* output = ReadInputString(prompt.String());
-    RESULT.Set(LispAtom::New(aEnvironment,aEnvironment.HashTable().LookUpStringify(output)->String()));
+    InternalUnstringify(prompt, promptObject->String());
+    char* output = ReadInputString(prompt.c_str());
+    RESULT = (LispAtom::New(aEnvironment,aEnvironment.HashTable().LookUpStringify(output)->c_str()));
 }
 
 static void LispHistorySize(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -345,7 +277,7 @@ void LispTime(LispEnvironment& aEnvironment, LispInt aStackTop)
 #else
     sprintf(buf,"%g",timeDiff);
 #endif
-    RESULT.Set(LispAtom::New(aEnvironment,buf));
+    RESULT = (LispAtom::New(aEnvironment,buf));
 }
 
 
@@ -353,14 +285,13 @@ void LispTime(LispEnvironment& aEnvironment, LispInt aStackTop)
 
 void LispFileSize(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    LispPtr fnameObject;
-    fnameObject.Set(ARGUMENT(1).Get());
+    LispPtr fnameObject = (ARGUMENT(1));
     CHK_ISSTRING_CORE(fnameObject,1);
     LispString fname;
-    InternalUnstringify(fname, fnameObject.Get()->String());
+    InternalUnstringify(fname, fnameObject->String());
 
     long fileSize = 0;
-    FILE* f = fopen(fname.String(),"rb");
+    FILE* f = fopen(fname.c_str(),"rb");
     if(f != NULL)
     {
       fseek(f,0,SEEK_END);
@@ -369,16 +300,16 @@ void LispFileSize(LispEnvironment& aEnvironment, LispInt aStackTop)
     }
     else
     {
-      printf("[[%s]]\n",fname.String());
-    }    
+      printf("[[%s]]\n",fname.c_str());
+    }
     char buf[100];
-    
+ 
 #ifdef HAVE_VSNPRINTF
     snprintf(buf,100,"%ld",fileSize);
 #else
     sprintf(buf,"%ld",fileSize);
 #endif
-    RESULT.Set(LispAtom::New(aEnvironment,buf));
+    RESULT = (LispAtom::New(aEnvironment,buf));
 }
 
 
@@ -400,9 +331,10 @@ void my_exit(void)
     // deleting the Yacas environment object, at least we have a saved history
     if (commandline) delete commandline; commandline = NULL;
     if (yacas) delete yacas; yacas = NULL;
+ 
     ReportNrCurrent();
 #ifdef YACAS_DEBUG
-    YacasCheckMemory();
+    CheckAllPtrs(1);
 #endif
   }
 #ifdef WIN32
@@ -501,7 +433,7 @@ void ShowResult(char *prompt)
     }
     else
     {
-        if ((*yacas)()().PrettyPrinter()==NULL)
+        if (yacas->getDefEnv().getEnv().PrettyPrinter()==NULL)
             printf("%s%s\n",prompt,yacas->Result());
     }
     if (use_texmacs_out)
@@ -533,30 +465,17 @@ void DeclarePath(char *ptr2)
   yacas->Evaluate(buf);
 }
 
-void DeclareDllPath(char *ptr2)
-{
-  char buf[1000];
-  if (ptr2[strlen(ptr2)-1] != PATH_SEPARATOR)
-    sprintf(buf,"DllDirectory(\"%s%s\");",ptr2,PATH_SEPARATOR_2);
-  else
-    sprintf(buf,"DllDirectory(\"%s\");",ptr2);
-  yacas->Evaluate(buf);
-}
-
 void LoadYacas(LispOutput* aOutput=NULL)
 {
+  if (yacas) return;
   busy=LispTrue;
   restart=LispFalse;
 
-  if (aOutput)
-    yacas = CYacas::NewL(aOutput,stack_size);
-  else
-    yacas = CYacas::NewL(stack_size);
+  yacas = CYacas::NewL(aOutput,stack_size);
 
 
-#define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) (*yacas)()().SetCommand(fname,iname,nrargs,flags);
+#define CORE_KERNEL_FUNCTION(iname,fname,nrargs,flags) yacas->getDefEnv().getEnv().SetCommand(fname,iname,nrargs,flags);
 
-CORE_KERNEL_FUNCTION("OSVersion",LispPlatformOS,0,YacasEvaluator::Function | YacasEvaluator::Fixed)
 CORE_KERNEL_FUNCTION("Exit",LispExit,0,YacasEvaluator::Function | YacasEvaluator::Fixed)
 CORE_KERNEL_FUNCTION("IsExitRequested",LispExitRequested,0,YacasEvaluator::Function | YacasEvaluator::Fixed)
 CORE_KERNEL_FUNCTION("HistorySize",LispHistorySize,1,YacasEvaluator::Function | YacasEvaluator::Fixed)
@@ -565,11 +484,10 @@ CORE_KERNEL_FUNCTION("IsPromptShown",LispIsPromptShown,0,YacasEvaluator::Functio
 CORE_KERNEL_FUNCTION("ReadCmdLineString",LispReadCmdLineString,1,YacasEvaluator::Function | YacasEvaluator::Fixed)
 CORE_KERNEL_FUNCTION("GetTime",LispTime,1,YacasEvaluator::Macro | YacasEvaluator::Fixed)
 CORE_KERNEL_FUNCTION("FileSize",LispFileSize,1,YacasEvaluator::Function | YacasEvaluator::Fixed)
-CORE_KERNEL_FUNCTION("PluginsCanBeLoaded",LispPluginsCanBeLoaded,0,YacasEvaluator::Function | YacasEvaluator::Fixed)
 
 
 #undef CORE_KERNEL_FUNCTION
-    
+ 
 
     if (archive)
     {
@@ -591,7 +509,7 @@ CORE_KERNEL_FUNCTION("PluginsCanBeLoaded",LispPluginsCanBeLoaded,0,YacasEvaluato
                     NEW CCompressedArchive(fullbuf, fullsize, compressed_archive);
                 if (a->iFiles.IsValid())
                 {
-                    (*yacas)()().iArchive = a;
+                    yacas->getDefEnv().getEnv().iArchive = a;
                 }
                 else
                 {
@@ -608,7 +526,7 @@ CORE_KERNEL_FUNCTION("PluginsCanBeLoaded",LispPluginsCanBeLoaded,0,YacasEvaluato
     }
 
     {
-        /* Split up root_dir in pieces separated by colons, and run 
+        /* Split up root_dir in pieces separated by colons, and run
            DefaultDirectory on each of them. */
         char *ptr1, *ptr2;
         ptr1 = ptr2 = root_dir;
@@ -626,7 +544,7 @@ CORE_KERNEL_FUNCTION("PluginsCanBeLoaded",LispPluginsCanBeLoaded,0,YacasEvaluato
         {
           yacas->Evaluate("Set(LoadPlugIns,False);");
         }
-        
+ 
         char buf[1000];
 #ifdef HAVE_VSNPRINTF
         snprintf(buf,1000,"Load(\"%s\");",init_script);
@@ -641,123 +559,13 @@ CORE_KERNEL_FUNCTION("PluginsCanBeLoaded",LispPluginsCanBeLoaded,0,YacasEvaluato
         }
     }
 
-    {
-        /* Split up dll_dir in pieces separated by colons, and run 
-           DllDirectory on each of them. */
-        char *ptr1, *ptr2;
-        ptr1 = ptr2 = dll_dir;
-        while (*ptr1 != '\0') {
-            while (*ptr1 != '\0' && *ptr1 != ':') ptr1++;
-            if (*ptr1 == ':') {
-                *ptr1 = '\0';
-                DeclareDllPath(ptr2);
-                ptr1++;
-                ptr2 = ptr1;
-            }
-        }
-        DeclareDllPath(ptr2);
-    }
-
     if (yacas->IsError())
         ShowResult("");
 
-    /* renaming .cc to .cpp files (I know, doesn't belong here ;-) ) */
-    /*
-    {
-        DIR *dp;
-        struct dirent* entry;
-        struct stat statbuf;
-        char dir[256];
-        strcpy(dir,"/root/myprojects/yacas-latest/src/");
-        if ((dp = opendir(dir)) != NULL)
-        {
-            while ((entry = readdir(dp)) != NULL)
-            {
-                memset(&statbuf,0,sizeof(struct stat));
-                stat(entry->d_name,&statbuf);
-                if (S_ISDIR(statbuf.st_mode))
-                    continue;
-                char *ext;
-                ext = strstr(entry->d_name,".cc");
-                if (ext==NULL)
-                    continue;
-                if (strcmp(ext,".cc"))
-                    continue;
-                char dummy[256];
-                strcpy(dummy,entry->d_name);
-                strstr(dummy,".cc")[0] = '\0';
-                char cmd[256];
-                sprintf(cmd,"mv %s.cc %s.cpp",dummy,dummy);
-                system(cmd);
-            }
-        }
-    }
-    return;
-*/
-    //Loading user addons
     if (use_texmacs_out)
     {
         printf("%cverbatim:",TEXMACS_DATA_BEGIN);
     }
-#ifndef WIN32 //TODO fix, with the file scanner in ramscripts
-    if (root_dir != NULL && root_dir[0] != '\0')
-    {
-        DIR *dp;
-        struct dirent* entry;
-        struct stat statbuf;
-        char dir[256];
-        char cwd[256];
-#ifdef HAVE_VSNPRINTF
-        snprintf(dir,256,"%saddons/",root_dir);
-#else
-        sprintf(dir,"%saddons/",root_dir);
-#endif
-
-        if ((dp = opendir(dir)) != NULL)
-        {
-            {
-                char ld[256];
-#ifdef HAVE_VSNPRINTF
-                snprintf(ld,256,"DefaultDirectory(\"%saddons/\");",root_dir);
-#else
-                sprintf(ld,"DefaultDirectory(\"%saddons/\");",root_dir);
-#endif
-                yacas->Evaluate(ld);
-            }
-            getcwd(cwd,256);
-            chdir(dir);
-            while ((entry = readdir(dp)) != NULL)
-            {
-                memset(&statbuf,0,sizeof(struct stat));
-                stat(entry->d_name,&statbuf);
-                if (S_ISDIR(statbuf.st_mode))
-                    continue;
-                char *ext;
-                ext = strstr(entry->d_name,".def");
-                if (ext==NULL)
-                    continue;
-                if (strcmp(ext,".def"))
-                    continue;
-                char buf[512];
-                char dummy[256];
-                strcpy(dummy,entry->d_name);
-                strstr(dummy,".def")[0] = '\0';
-                if (show_prompt && !use_texmacs_out)
-                    printf("[%s] ",dummy);
-#ifdef HAVE_VSNPRINTF
-                snprintf(buf,512,"DefLoad(\"%s\");",dummy);
-#else
-                sprintf(buf,"DefLoad(\"%s\");",dummy);
-#endif
-                yacas->Evaluate(buf);
-            }
-            closedir(dp);
-            chdir(cwd);
-            if (show_prompt)
-                printf("\n");
-        }
-    }
-#endif
     {
         char fname[256];
 #ifdef HAVE_VSNPRINTF
@@ -791,7 +599,7 @@ void InterruptHandler(int errupt)
 #endif
 {
     printf("^C pressed\n");
-    (*yacas)()().iEvalDepth = (*yacas)()().iMaxEvalDepth+100;
+    yacas->getDefEnv().getEnv().iEvalDepth = yacas->getDefEnv().getEnv().iMaxEvalDepth+100;
     if (readmode)
     {
       my_exit();
@@ -810,7 +618,7 @@ void BusErrorHandler(int errupt)
 #endif
 
     printf("Bus error/segfault\n");
-    (*yacas)()().iEvalDepth = (*yacas)()().iMaxEvalDepth+100;
+    yacas->getDefEnv().getEnv().iEvalDepth = yacas->getDefEnv().getEnv().iMaxEvalDepth+100;
 }
 */
 
@@ -830,7 +638,7 @@ VOID CALLBACK stopClient(LPVOID lpArgToCompletionRoutine, DWORD dwTimerLowValue,
 {
   if (clientToStop)
   {
-    (*clientToStop)()().iEvalDepth = (*clientToStop)()().iMaxEvalDepth+100;
+    clientToStop->getDefEnv().getEnv().iEvalDepth = clientToStop->getDefEnv().getEnv().iMaxEvalDepth+100;
   }
 }
 
@@ -843,7 +651,7 @@ int runserver(int argc,char** argv)
    if (hideconsolewindow)
    {
       // format a "unique" newWindowTitle
-      char newWindowTitle[256];   
+      char newWindowTitle[256];
       wsprintf(newWindowTitle,"%d/%d", GetTickCount(), GetCurrentProcessId());
       // change current window title
       SetConsoleTitle(newWindowTitle);
@@ -858,7 +666,7 @@ int runserver(int argc,char** argv)
       }
    }
 #endif
-   
+ 
    int server_sockfd, client_sockfd;
     SOCKLEN_T server_len, client_len;
     struct sockaddr_in server_address;
@@ -875,6 +683,8 @@ int runserver(int argc,char** argv)
 
 
     int seconds = 30; // give each calculation only so many calculations
+    if (server_single_user)
+      seconds = 0;
 
     serverbusy=1;
     maxConnections=MAX_CONNECTIONS;
@@ -886,7 +696,7 @@ int runserver(int argc,char** argv)
     WSADATA wsadata;
 
     if (WSAStartup(0x101, &wsadata))
-    {  
+    {
        perror("YacasServer Could not initiate Winsock DLL\n");
        exit(1);
     }
@@ -905,7 +715,7 @@ int runserver(int argc,char** argv)
         if (setsockopt(server_sockfd,SOL_SOCKET,SO_REUSEADDR,(const char *)&rsp,sizeof(int)))
 #endif
         {
-            perror("YacasServer Could not set socket options\n");            
+            perror("YacasServer Could not set socket options\n");
             exit(1);
         }
     }
@@ -919,15 +729,15 @@ int runserver(int argc,char** argv)
 
     if (bind(server_sockfd, (struct sockaddr *)&server_address, server_len))
     {
-        perror("YacasServer Could not bind to the socket\n");        
+        perror("YacasServer Could not bind to the socket\n");
         exit(1);
     }
     if (listen(server_sockfd, maxConnections))
     {
-        perror("YacasServer Could not listen to the socket\n");        
+        perror("YacasServer Could not listen to the socket\n");
         exit(1);
     }
-    
+ 
     {
       int i;
       for (i=0;i<MAX_CONNECTIONS;i++)
@@ -944,20 +754,20 @@ int runserver(int argc,char** argv)
         int nread;
         testfds = readfds;
 
-        result = select(FD_SETSIZE, &testfds, (fd_set *)0, 
+        result = select(FD_SETSIZE, &testfds, (fd_set *)0,
                         (fd_set *)0, (struct timeval *) 0);
 
         if(result < 1)
         {
-            perror("server5");            
+            perror("server5");
             exit(1);
         }
-        
+ 
 #ifndef WIN32
         int socketcount = FD_SETSIZE;
 #else
         int socketcount = readfds.fd_count;
-#endif        
+#endif
         for(int sockindex = 0; sockindex < socketcount; sockindex++)
         {
 #ifndef WIN32
@@ -1000,13 +810,20 @@ int runserver(int argc,char** argv)
                     {
 #ifndef WIN32
                         close(fd);
-#else   
+#else
                         closesocket(fd);
 #endif
                         FD_CLR(fd, &readfds);
                         delete used_clients[clsockindex];
                         used_clients[clsockindex] = NULL;
                         nrSessions--;
+ 
+                        if (server_single_user && nrSessions == 0)
+                        {
+                          exit(0);
+                        }
+
+ 
 #ifdef YACAS_DEBUG
                         printf("Removing client on %d\n",fd);
 #endif
@@ -1016,34 +833,34 @@ int runserver(int argc,char** argv)
 //                        LispString cmd;
 //                        cmd.SetNrItems(0);
                         char* buffer = (char*)malloc(nread+1);
-                        if (!buffer) 
+                        if (!buffer)
                         {
-                          printf("Error: seem to have run out of memory\n");                          
+                          printf("Error: seem to have run out of memory\n");
                           exit(-1);
                         }
-                        
+ 
                         int finalbuffersize = BUFFER_CHUNKSIZE;
                         char* finalbuffer = (char*)malloc(finalbuffersize);
-                        if (!finalbuffer) 
+                        if (!finalbuffer)
                         {
-                          printf("Error: seem to have run out of memory\n");                          
+                          printf("Error: seem to have run out of memory\n");
                           exit(-1);
-                        }                
-                        
+                        }
+ 
                         finalbuffer[0] = '\0';
 
                         int bytesread;
                         int totalbytesread = 0;
-                        
+ 
 #ifndef WIN32
                         while((bytesread = read(fd, buffer, nread)) != 0)
-#else                        
+#else
                         while(bytesread = recv(fd, buffer, nread, 0))
-#endif                   
-	                     {
+#endif
+                       {
                            buffer[bytesread] = '\0';
                            totalbytesread += bytesread;
-                           
+ 
                            if (totalbytesread >= finalbuffersize)
                            {
                               // If input string has exeeded the buffer size reallocate the buffer
@@ -1052,48 +869,48 @@ int runserver(int argc,char** argv)
                                 finalbuffersize = totalbytesread;
 //                              finalbuffersize = max(totalbytesread, finalbuffersize + BUFFER_CHUNKSIZE);
                               char* newfinalbuffer = (char*)malloc(finalbuffersize);
-                              if (!newfinalbuffer) 
+                              if (!newfinalbuffer)
                               {
-                                printf("Error: seem to have run out of memory\n");                          
+                                printf("Error: seem to have run out of memory\n");
                                 exit(-1);
                               }
 
-                              strcpy(newfinalbuffer, finalbuffer); 
+                              strcpy(newfinalbuffer, finalbuffer);
 
                               free(finalbuffer);
                               finalbuffer = newfinalbuffer;
                            }
-                           
+ 
                            // Append current buffer to finalbuffer string
                            strncat(finalbuffer, buffer, bytesread);
 
                            // Check for semicolon terminator
                            char * ptr;
                            if ((ptr = strchr(finalbuffer, ';')) != NULL)
-                           {                              
+                           {
                               *ptr = '\0';
                               break;
                            }
-                        }                        
+                        }
                         free(buffer);
 /*
                         int i;
-                        for (i=0;i<nread;i++) 
+                        for (i=0;i<nread;i++)
                         {
                           cmd.Append(buffer[i]);
                         }
                         free(buffer);
-                        cmd.Append('\0');                        
+                        cmd.Append('\0');
 
 */
 #ifdef YACAS_DEBUG
 printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
 #endif
-                            
+ 
                             if (clsockindex < maxConnections)
                             {
                                if (used_clients[clsockindex] == NULL)
-                               {                                                                                      
+                               {
 
    #ifdef YACAS_DEBUG
      printf("Loading new Yacas environment\n");
@@ -1109,7 +926,10 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
    //                        if (fork() == 0)
                            {
                                char* response = finalbuffer;
-                               (*used_clients[clsockindex])()().iSecure = 1;
+                               if (!server_single_user)
+                               {
+                                 used_clients[clsockindex]->getDefEnv().getEnv().iSecure = 1;
+                               }
                                if (seconds>0)
                                {
                                    clientToStop = used_clients[clsockindex];
@@ -1119,18 +939,24 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
    #else
                                    LARGE_INTEGER timedue;
                                    timedue.QuadPart = seconds  * -10000000;
-                                
+ 
                                    htimer = CreateWaitableTimer(NULL, true, "WaitableTimer");
                                    SetWaitableTimer(htimer, &timedue, 0, stopClient, NULL, 0);
-   #endif                                
+   #endif
                                }
    #ifdef YACAS_DEBUG
                                printf("In> %s\n",finalbuffer);
    #endif
-                               outStrings.SetNrItems(1);
+                               outStrings.ResizeTo(1);
                                outStrings[0] = '\0';
                                used_clients[clsockindex]->Evaluate(finalbuffer);
                                free(finalbuffer);
+
+                                if (server_single_user && !Busy())
+                                {
+                                  exit(0);
+                                }
+
                                if (seconds>0)
                                {
    #ifndef WIN32
@@ -1154,13 +980,13 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
 
                                int buflen=strlen(response);
    #ifdef YACAS_DEBUG
-                               printf("%s",outStrings.String());
+                               printf("%s",outStrings.c_str());
                                printf("Out> %s\n",response);
    #endif
                                if (response)
                                {
    #ifndef WIN32
-                                 write(fd, outStrings.String(), strlen(outStrings.String()));
+                                 write(fd, outStrings.c_str(), strlen(outStrings.c_str()));
                                  write(fd,"]\r\n",3);
                                  if (buflen>0)
                                  {
@@ -1169,7 +995,7 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
                                  }
                                  write(fd,"]\r\n",3);
    #else
-                                 send(fd, outStrings.String(), strlen(outStrings.String()), 0);
+                                 send(fd, outStrings.c_str(), strlen(outStrings.c_str()), 0);
                                  send(fd,"]\r\n",3, 0);
                                  if (buflen>0)
                                  {
@@ -1179,7 +1005,7 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
                                  send(fd,"]\r\n",3, 0);
    #endif
                                }
-                           }                        
+                           }
 
    // enable if fork needed
    //                            exit(0);
@@ -1195,7 +1021,7 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
                            send(fd,"]\r\n",3, 0);
                            send(fd, limtxt, strlen(limtxt), 0);
                            send(fd,"]\r\n",3, 0);
-#endif                          
+#endif
                         }
                     }
                 }
@@ -1213,7 +1039,7 @@ printf("Servicing on %ld (%ld)\n",(long)fd,(long)used_clients[clsockindex]);
 
 
 
-#define SHOWSIZE(a)    printf("   sizeof(" #a ") = %ld\n",sizeof(a));
+#define SHOWSIZE(a)    printf("   sizeof(" #a ") = %ld\n",(long)sizeof(a));
 
 int main(int argc, char** argv)
 {
@@ -1239,30 +1065,32 @@ int main(int argc, char** argv)
   }
 #endif
 
-    
+ 
 #ifdef YACAS_DEBUG
 //        PlatAlloc(100); // test the alloc memory leak checker
     CHECKPTR(NULL);
 #endif
-    
+ 
 /*
     printf("sizeof(LispAtom) = %d\n",sizeof(LispAtom));
     printf("sizeof(LispSubList) = %d\n",sizeof(LispSubList));
     printf("sizeof(LispString) = %d\n",sizeof(LispString));
 */
-    
+
+#define USE_TEXMACS_OUT yacas->getDefEnv().getEnv().SetPrettyPrinter(yacas->getDefEnv().getEnv().HashTable().LookUp("\"TexForm\""));
+
+ 
     char* file_to_load=NULL;
-    
+    bool exit_after_files = false;
+
     int fileind=1;
     if (argc > 1)
     {
-        while (fileind<argc && argv[fileind][0] == '-')
+        for ( ; fileind<argc && argv[fileind][0] == '-'; fileind++)
         {
             if (!strcmp(argv[fileind],"--texmacs"))
             {
                 use_texmacs_out = 1;
-//                show_prompt=0;
-                trace_history=1;
                 use_plain = 1;
                 read_eval_print = NULL;
             }
@@ -1287,7 +1115,6 @@ int main(int argc, char** argv)
             else if (!strcmp(argv[fileind],"--read-eval-print"))
             {
                 fileind++;
-
 //printf("rep %d\n",argv[fileind][0]);
                 if (fileind<argc)
                 {
@@ -1299,17 +1126,11 @@ int main(int argc, char** argv)
             }
             else if (!strcmp(argv[fileind],"--rootdir"))
             {
-                archive = NULL; 
+                archive = NULL;
                 fileind++;
                 if (fileind<argc)
                   root_dir = argv[fileind];
             }
-            else if (!strcmp(argv[fileind],"--dlldir"))
-            {
-                fileind++;
-                if (fileind<argc)
-                  dll_dir = argv[fileind];
-            }
             else if (!strcmp(argv[fileind],"--archive"))
             {
                 fileind++;
@@ -1317,41 +1138,34 @@ int main(int argc, char** argv)
                   archive = argv[fileind];
             }
             else if (!strcmp(argv[fileind],"--server"))
-            {    
+            {
                 fileind++;
-
                 if (fileind<argc)
                 {
-                  server_mode=1;
-                  server_port = atoi(argv[fileind]);
-                }                
+                    server_mode=1;
+                    server_port = atoi(argv[fileind]);
+                }
+            }
+            else if (!strcmp(argv[fileind],"--single-user-server"))
+            {
+                server_single_user = 1;
             }
-
-
             else if (!strcmp(argv[fileind],"--stacksize"))
-            {    
+            {
                 fileind++;
-
                 if (fileind<argc)
                 {
                   stack_size = atoi(argv[fileind]);
-                }                
+                }
             }
-
-
             else if (!strcmp(argv[fileind],"--execute"))
-            {    
+            {
                 fileind++;
-
                 if (fileind<argc)
                 {
                   execute_commnd = argv[fileind];
-                }                
+                }
             }
-
-
-
-
             else if (!strcmp(argv[fileind],"--uncompressed-archive"))
             {
                 // This is just test code, to see if the engine
@@ -1369,6 +1183,26 @@ int main(int argc, char** argv)
             {
               compiled_plugins = 0;
             }
+      else if (!strcmp(argv[fileind],"-i"))
+      {
+        fileind++;
+        if (fileind<argc)
+        {
+          char* immediate = argv[fileind];
+          if (immediate)
+          {
+            LoadYacas();
+            if (use_texmacs_out) USE_TEXMACS_OUT;
+            yacas->Evaluate(immediate);
+            if (yacas->IsError())
+            {
+              printf("Error in immediate command %s:\n",immediate);
+              printf("%s\n",yacas->Error());
+            }
+            exit_after_files = true;
+          }
+        }
+            }
             else
             {
                 if (strchr(argv[fileind],'f'))
@@ -1385,7 +1219,7 @@ int main(int argc, char** argv)
                 }
                 if (strchr(argv[fileind],'t'))
                 {
-                    trace_history=1;
+                  // Not sure what we should do with -t ... it used to be trace_history
                 }
                 if (strchr(argv[fileind],'d'))
                 {
@@ -1396,7 +1230,7 @@ int main(int argc, char** argv)
                 {
                     hideconsolewindow=1;
                 }
-              
+ 
 #ifdef HAVE_CONFIG_H
                 if (strchr(argv[fileind],'v'))
                 {
@@ -1424,7 +1258,6 @@ int main(int argc, char** argv)
                 }
 #endif
             }
-            fileind++;
         }
         if (fileind<argc)
             file_to_load=argv[fileind];
@@ -1470,9 +1303,6 @@ int main(int argc, char** argv)
         commandline = NEW FANCY_COMMAND_LINE;
 #endif
 
-
-    commandline->iTraceHistory = trace_history;
-
     LoadYacas();
 
 
@@ -1485,13 +1315,12 @@ int main(int argc, char** argv)
 
     if (use_texmacs_out)
     {
-        (*yacas)()().SetPrettyPrinter((*yacas)()().HashTable().LookUp("\"TexForm\""));
-//        yacas->Evaluate("ToString()PrettyPrinter(\"TexForm\");");
+        USE_TEXMACS_OUT;
     }
 
-    if (fileind<argc)
+ 
     {
-        while (fileind<argc)
+        for ( ; fileind<argc; fileind++)
         {
             char s[200];
             if (patchload)
@@ -1517,11 +1346,16 @@ int main(int argc, char** argv)
                 printf("Error in file %s:\n",argv[fileind]);
                 printf("%s\n",yacas->Error());
             }
-            fileind++;
+      exit_after_files = true;
         }
-        my_exit();
-        exit(0);
     }
+
+    if (exit_after_files)
+  {
+    my_exit();
+        exit(0);
+  }
+
     if (show_prompt && (!use_texmacs_out))
         ShowResult("");
     if (execute_commnd != NULL)
@@ -1563,7 +1397,7 @@ RESTART:
             printf(GPL_blurb_nohelp);
             printf("%c",TEXMACS_DATA_END);
         }
-        else 
+        else
         {
             printf("This is Yacas version '" VERSION "'.\n");
 #ifdef WIN32
@@ -1571,9 +1405,7 @@ RESTART:
 #else
             printf(GPL_blurb);
 #endif
-            printf("Numeric mode: \"%s\"\n",NumericLibraryName());
-            printf("To exit Yacas, enter  Exit(); or quit or Ctrl-c. Type ?? for help.\n");
-            printf("Or type ?function for help on a function.\n");
+            printf("To exit Yacas, enter  Exit(); or quit or Ctrl-c.\n");
             printf("Type 'restart' to restart Yacas.\n");
             printf("To see example commands, keep typing Example();\n");
         }
@@ -1585,7 +1417,7 @@ RESTART:
       while (Busy())
       {
 #ifdef YACAS_DEBUG
-        LispLocalEvaluator local((*yacas)()(),NEW TracedStackEvaluator);
+        LispLocalEvaluator local(yacas->getDefEnv().getEnv(),NEW TracedStackEvaluator);
 #endif
         yacas->Evaluate(read_eval_print);
         if (yacas->IsError())
@@ -1599,9 +1431,9 @@ RESTART:
       while (Busy())
       {
 #ifdef YACAS_DEBUG
-        LispLocalEvaluator local((*yacas)()(),NEW TracedStackEvaluator);
+        LispLocalEvaluator local(yacas->getDefEnv().getEnv(),NEW TracedStackEvaluator);
 #endif
-        
+ 
 #ifdef PROMPT_SHOW_FREE_MEMORY
         char full_prompt[30];
         if (show_prompt)
@@ -1612,7 +1444,7 @@ RESTART:
 #else
         ReadInputString(inprompt);
 #endif
-        char *inpline =  commandline->iLine.String();
+        char *inpline =  commandline->iLine.c_str();
         if (use_texmacs_out)
         {
             printf("%cverbatim:",TEXMACS_DATA_BEGIN);
@@ -1652,7 +1484,7 @@ RESTART:
   }
   if (restart)
   {
-    delete yacas;
+    delete yacas; yacas = NULL;
     LoadYacas();
     goto RESTART;
   }
